@@ -1,7 +1,6 @@
 from typing import Optional, List, Dict, Any
 
-# noinspection PyProtectedMember
-from pyspark import keyword_only
+from spark_pipeline_framework.utilities.capture_parameters import capture_parameters
 from pyspark.ml.param import Param
 from pyspark.sql.dataframe import DataFrame
 
@@ -14,7 +13,7 @@ from spark_pipeline_framework.transformers.framework_transformer.v1.framework_tr
 
 class FrameworkSelectColumnsTransformer(FrameworkTransformer):
     # noinspection PyUnusedLocal
-    @keyword_only
+    @capture_parameters
     def __init__(
         self,
         view: Optional[str] = None,
@@ -54,8 +53,8 @@ class FrameworkSelectColumnsTransformer(FrameworkTransformer):
         # name = self.getName()
         view = self.getView()
         # progress_logger: ProgressLogger = self.getProgressLogger()
-        drop_columns: Optional[List[str]] = self.getOrDefault("drop_columns")
-        keep_columns: Optional[List[str]] = self.getOrDefault("keep_columns")
+        drop_columns: Optional[List[str]] = self.getOrDefault(self.drop_columns)
+        keep_columns: Optional[List[str]] = self.getOrDefault(self.keep_columns)
 
         assert view
         result_df: DataFrame = df.sql_ctx.table(view)
